/***********************************************************
PROJECT  : bit-banging i2c interface library
MODULE   : Libi2c on serial port
TARGET   : Linux/x86
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sat Mar  1 15:42:44 MET 1997 on Tom
UPDATE   : Wed Apr 10 13:10:32 CEST 2002 on Idefix
VERSION  : 1.6
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
/* the serial access layer */
#include "serial/serial.h"
/* prototypes */
#include "libi2c-bb.h"
/* global library interface */
#include "libi2c.h"

static int debug;

static u_char sda,scl;

/* I2C_xxx are internal (static) functions */
/* i2c_xxx are external exported functions */
/* debug>0 will debug i2c_xxx */
/* debug>1 will debug I2C_xxx (but not bus waits) */
/* debug>2 will also debug bus waits */ 

/* Wait till SCL is released */
static u_short I2C_waitCL(void) {
	u_char dat,cl=0;
	long timeout=TIMEOUT;
	if (debug>3) fprintf(stderr," I2C_waitCL\n");
	while ((cl == 0) && (timeout)) {
	   GetSCLSDA(&cl,&dat);timeout--;
        }
        if (timeout) return I2C_OK;
	else {
	       if (debug>1) fprintf(stderr,"** timeout : I2C_waitCL\n");
		return I2C_TIMEOUT;
	}
}

/* Wait till i2c bus free */
static u_short I2C_waitfree(void) {
	u_char dat=0,cl=0;
	long timeout=TIMEOUT;
	if (debug>3) fprintf(stderr," I2C_waitfree\n");
	while (((cl == 0) || (dat == 0)) && (timeout)) {
	   GetSCLSDA(&cl,&dat);timeout--;
	}
        if (timeout) return I2C_OK;
	else {
	       if (debug>1) fprintf(stderr,"** timeout : I2C_waitfree\n");
		return I2C_TIMEOUT;
	}
}

/* End i2c trnsaction */
static u_short I2C_end(void) {
        short i;
	if (debug>2) fprintf(stderr," I2C_end\n");
	sda=0;       SetSCLSDA(scl,sda);
	       scl=1;SetSCLSDA(scl,sda);
	if (I2C_waitCL()) return I2C_TIMEOUT;
	sda=1;       SetSCLSDA(scl,sda);
	for(i=0;i<7;i++) {
               scl=0;SetSCLSDA(scl,sda);
               scl=1;SetSCLSDA(scl,sda);
	   if (I2C_waitCL()) return I2C_TIMEOUT;
	}
        return I2C_OK;
}

/* Start i2c transaction */
static u_short I2C_start(void) {
	if (debug>2) fprintf(stderr," I2C_start\n");
        sda=1; scl=1;SetSCLSDA(scl,sda);
        if (I2C_waitfree()) return I2C_TIMEOUT;
        sda=0;       SetSCLSDA(scl,sda);
               scl=0;SetSCLSDA(scl,sda);
        return I2C_OK;
}

/* Send stop on i2c bus */
static u_short I2C_stop(void) {
	if (debug>2) fprintf(stderr," I2C_stop\n");
	sda=0;	     SetSCLSDA(scl,sda);
	       scl=1;SetSCLSDA(scl,sda);
	if (I2C_waitCL()) return I2C_TIMEOUT;
	sda=1;       SetSCLSDA(scl,sda);
        return I2C_OK;
}

/* Write a byte on the bus */
static u_short I2C_wbyte(u_char x) {
	u_char dat,cl;
	u_char i,j;
	if (debug>2) fprintf(stderr," I2C_wbyte(0x%02X)\n",x);
	j=128;
	for(i=0;i<=7;i++) {
		sda=((x&j)!= 0);SetSCLSDA(scl,sda);
			 scl=1;SetSCLSDA(scl,sda);
		if (I2C_waitCL()) return I2C_TIMEOUT;
			 scl=0;SetSCLSDA(scl,sda);
		j= j>>1;
	}
	sda=1;		SetSCLSDA(scl,sda);
		  scl=1;SetSCLSDA(scl,sda);
	if (I2C_waitCL()) return I2C_TIMEOUT;
	GetSCLSDA(&cl,&dat);
		  scl=0;SetSCLSDA(scl,sda);
	return dat;
}

/* Send ack */
static u_short I2C_ack(void) {
	if (debug>2) fprintf(stderr," I2C_ack\n");
		  scl=0;SetSCLSDA(scl,sda);
	sda=0;		SetSCLSDA(scl,sda);
		  scl=1;SetSCLSDA(scl,sda);
	if (I2C_waitCL()) return I2C_TIMEOUT;
		  scl=0;SetSCLSDA(scl,sda);
        return I2C_OK;
}

/* Send nack */
static u_short I2C_nack(void) {
	if (debug>2) fprintf(stderr," I2C_nack\n");
		  scl=0;SetSCLSDA(scl,sda);
	sda=1;		SetSCLSDA(scl,sda);
		  scl=1;SetSCLSDA(scl,sda);
	if (I2C_waitCL()) return I2C_TIMEOUT;
		  scl=0;SetSCLSDA(scl,sda);
        return I2C_OK;
}

/* Read a byte on the bus */
static u_short I2C_rbyte(u_char *x) {
	u_char dat,cl;
	u_char i;
	*x=0;
	for(i=0;i<=7;i++) {
		sda=1;scl=0;SetSCLSDA(scl,sda);
		      scl=1;SetSCLSDA(scl,sda);
		if (I2C_waitCL()) return I2C_TIMEOUT;
		GetSCLSDA(&cl,&dat);
		*x=((*x)<<1)+(dat>0);
	}
	if (debug>1) fprintf(stderr," I2C_rbyte -> 0x%02X \n",*x);
	return 1;
}

/* ****************** */
/* Exported functions */
/* ****************** */

/* Init interface */
u_short i2c_init(int verbose) {
    int res;
    debug=verbose;
    if (debug>1) fprintf(stderr,"[i2c_init]\n");
    res=OpenSerial(SERDEV);
    if (res<0) {
        fprintf(stderr,"* error : i2c_init (device error)\n");
        return I2C_OPENERROR;
    }
    sda=scl=1;
    SetSCLSDA(scl,sda);
    usleep(100000); /* StartUp delay (& powerup if self powered) */
    I2C_wbyte(0xFF);
    I2C_end();
    if (I2C_waitfree()) {
        fprintf(stderr,"* error : i2c_init (bus error)\n");
        return I2C_TIMEOUT;
    }
    return I2C_OK;
}

/* Interface has no address */
u_short i2c_adr(u_char address) {
//    u_short res;
    if (debug>1) fprintf(stderr,"[i2c_adr(0x%02X)]\n",address);
    return I2C_OK;
}

/* reinit (after failure) */
u_short i2c_reinit(void) {
    if (debug>1) fprintf(stderr,"[i2c_reinit]\n");
    sda=scl=1;
    SetSCLSDA(scl,sda);
    I2C_wbyte(0);
    I2C_end();
    if (I2C_waitfree()) return I2C_TIMEOUT;
    if (debug) fprintf(stderr,"i2c_init\n");
    return 0;
}

/* Close interface */
void i2c_close(void) {
    if (debug>1) fprintf(stderr,"[i2c_close]\n");
    CloseSerial();
}

/* Send count byte from buffer to device at address */
u_short i2c_write(u_char address, u_char *buffer, u_char count) {
 u_short s,ack;

        if (debug>1) {
            fprintf(stderr,"[i2c_write(0x%02X)",address);
            for(s=0;s<count;s++) {
                fprintf(stderr," %02X",buffer[s]);
            }
            fprintf(stderr," ]\n");
        }
	I2C_start();
	ack=I2C_wbyte(address & 254);
	if (ack) {
		I2C_stop();
		fprintf(stderr,"Wrong address i2c_write(0x%02X)\n",address & 254);
		return ack;
	}
	for(s=0;s<count;s++) {
		ack=I2C_wbyte(buffer[s]);
		if (ack) {
			I2C_stop();
			fprintf(stderr,"Write error i2c_write(0x%02X)\n",address & 254);
			return ack;
		}
	}
	I2C_stop();
	if (ack && debug) fprintf(stderr,"* i2c_write, ack\n");
	return (ack<<8)|count;
}

/* Read count byte in buffer from device at address */
u_short i2c_read(u_char address, u_char *buffer, u_char count) {
 u_short s,ack;
        if (debug>1) fprintf(stderr,"[i2c_read(0x%02X)]\n",address);
	I2C_start();
	ack=I2C_wbyte((address & 254)+1);
	if (ack) {
		I2C_stop();
		if (debug) fprintf(stderr,"Wrong address i2c_read(0x%02X)\n",(address & 254)+1);
		return (ack<<8);
	}
	for(s=0;s<count;s++) {
		I2C_rbyte(&(buffer[s]));
		if (s<count-1) I2C_ack(); else I2C_nack();
	}
	I2C_stop();
        if (debug>1) {
            fprintf(stderr,"[          ->");
            for(s=0;s<count;s++) {
                fprintf(stderr," %02X",buffer[s]);
            }
            fprintf(stderr," ]\n");
        }
	if (ack&&debug) fprintf(stderr,"* i2c_read, ack\n");
	return (ack<<8)|count;
}

/* Test if a device respond at address */
/* for standard devices, this should not do anything else */
u_short i2c_test(u_char address) {
    u_short res=1;
    if (debug>1) fprintf(stderr,"[i2c_test(0x%02X) ",address);
    if (debug>2) fprintf(stderr,"]\n");
    I2C_start();
    res=I2C_wbyte((address & 254)+1);
    I2C_stop();
    if (res==I2C_TIMEOUT) {
        i2c_reinit();
    }
    if (debug>1) fprintf(stderr," %c\n",res==I2C_OK?'x':'_');
    return res;
}

/* Write countw bytes from buffer bufw then immediately read */
/* countr bytes in buffer bufr. Different from write followed by */
/* read since it is one transaction only */
u_short i2c_write_read(u_char address, u_char *bufw, u_char countw, \
    u_char *bufr, u_char countr) {
    u_short s,ack;
        if (debug>1) {
            fprintf(stderr,"[i2c_write_read(0x%02X)",address);
            for(s=0;s<countw;s++) {
                fprintf(stderr," %02X",bufw[s]);
            }
            fprintf(stderr," ]\n");
        }
	if (countw==0) return i2c_read(address,bufr,countr);
	if (countr==0) return i2c_write(address,bufw,countw);
	I2C_start();
	ack=I2C_wbyte(address & 254);
	if (ack) {
		I2C_stop();
		fprintf(stderr,"Wrong address i2c_write_read(0x%02X) write\n",address & 254);
		return (ack<<8);
	}
	for(s=0;s<countw;s++) {
		ack=I2C_wbyte(bufw[s]);
		if (ack) {
			I2C_stop();
			fprintf(stderr,"Write error i2c_write_read(0x%02X) write\n",address & 254);
			return (ack<<8)|s;
		}
	}
	I2C_start();
	ack=I2C_wbyte((address & 254)+1);
	if (ack) {
		I2C_stop();
		fprintf(stderr,"Wrong address i2c_write_read(0x%02X) read\n",(address & 254)+1);
		return (ack<<8)|countw;
	}
		
	for(s=0;s<countr;s++) {
		I2C_rbyte(&(bufr[s]));
		if (s<countr-1) I2C_ack(); else I2C_nack();
	}
	I2C_stop();
        if (debug>1) {
            fprintf(stderr,"[          ->");
            for(s=0;s<countr;s++) {
                fprintf(stderr," %02X",bufr[s]);
            }
            fprintf(stderr," ]\n");
        }
    if (ack&&debug) fprintf(stderr,"* i2c_write_read, ack\n");
	return (ack<<8)|countw;
}

/* lock and unlock exist only for compatibility with */
/* the pcf8584 driver library */
u_short i2c_lock(void) {
    if (debug) fprintf(stderr,"i2c_lock\n");
    return 0;
}

u_short i2c_unlock(void) {
    if (debug) fprintf(stderr,"i2c_unlock\n");
    return 0;
}
