/***********************************************************
PROJECT  : bit-banging i2c interface library
MODULE   : Linux serial port handling
TARGET   : Linux/x86
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sat Mar  1 15:42:44 MET 1997
UPDATE   : Thu Dec 16 23:20:53 CET 1999
VERSION  : 1.5
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>

static int sfd=-1;

/* Opening serial port */
int OpenSerial(char *dev) {
    u_char flags;
    sfd = open(dev,O_RDWR); 
#if defined(TIOCSBRK) && defined(TIOCCBRK)
    if ((ioctl(sfd,TIOCSBRK,0) == -1) || (ioctl(sfd,TIOCCBRK,0) == -1)) {
        perror("OpenSerial() ioctl() pb\n"); 
        return -1; 
    }
#else
#warning OpenSerial()
    close(fd);
    return -1; 
#endif

/* open sets RTS and DTR, reset it */
#if defined(TIOCMGET) && defined(TIOCMSET)
    if (ioctl(sfd,TIOCMGET, &flags)== -1) {
        perror("OpenSerial() ioctl() pb\n"); 
        return -1; 
    } else {
        flags &= ~(TIOCM_RTS|TIOCM_DTR);
	if (ioctl(sfd,TIOCMSET, &flags)== -1) {
            perror("OpenSerial() ioctl() pb\n"); 
	    return -1; 
	 }
    }
#else
#warning OpenSerial()
    close(sfd);
    return -1; 
#endif
    return 0;
}

/* Closing serial port */
int CloseSerial(void) {
    close(sfd);
    return 0;
}

int SetRTSDTR(u_char rts, u_char dtr) {
        int result; 
	int flags; 
	if (ioctl(sfd,TIOCMGET, &flags) == -1) {
	   return -1;
	}
    	if (rts)
		flags |= TIOCM_RTS; 
	else
		flags &= ~TIOCM_RTS; 

       if (dtr)
		flags |= TIOCM_DTR; 
	else
		flags &= ~TIOCM_DTR; 
       result = ioctl(sfd,TIOCMSET, &flags); 
       return result;
}

int GetCTSDSR(u_char *cts, u_char *dsr) {
 	int flags; 
	if (ioctl(sfd,TIOCMGET, &flags) == -1) {
            return -1;
        }
        *dsr=((flags & TIOCM_DSR) != 0);
        *cts=((flags & TIOCM_CTS) != 0);
        return 0;
}
