/***********************************************************
PROJECT  : bit-banging i2c interface library
MODULE   : Libi2c on serial port
TARGET   : Linux/x86
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sat Mar  1 15:42:44 MET 1997 on Tom
UPDATE   : Tue Apr  9 23:56:36 CEST 2002 on Avoranfix
VERSION  : 1.6
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

#ifndef _I2C_LIB_H
#define _I2C_LIB_H

#include <sys/types.h>

/* Error codes (Identical to COMM'NET ) */
/* low 8 bits should give back the number */
/* of transmitted bytes */
#define I2C_OK 0x0 /* transmission OK */
/* #define I2C_D_START 0x1000 */ /* double start */
#define I2C_WR_NOACK 0x2000 /* write, no ACK */
#define I2C_RD_NOACK 0x4800 /* read, no ACK */
#define I2C_WOK_NOACK 0x3000 /* data sent, no ACK */
/* #define I2C_ROK_NOACK 0x5800 */ /* data read, no ACK */
#define I2C_TIMEOUT 0x0100 /* timeout waiting for bus */
#define I2C_OPENERROR 0xFF00

/* Initialize bus */
u_short i2c_init(int verbose);

/* Test if a device exists on the bus */
u_short i2c_test(u_char address);

/* set self address (not used for bit-banging) */
u_short i2c_adr(u_char address);

/* reinitialize the bus (for error recovery) */
u_short i2c_reinit(void);

/* close bus */
void i2c_close(void);

/* send count data from buffer at address */
/* return in the low 8 bits the number of bytes corectly written */
u_short i2c_write(u_char address, u_char *buffer, u_char count);

/* read count data in buffer at address */
/* return in the low 8 bits the number of bytes read */
u_short i2c_read(u_char address, u_char *buffer, u_char count);

/* write countw data from bufw then read countr data in bufr at address */
/* return in the low 8 bits the number of bytes corectly written */
u_short i2c_write_read(u_char address, u_char *bufw, u_char countw,\
        u_char *bufr, u_char countr);

/* lock / unlock access to the bus */
u_short i2c_lock(void);
u_short i2c_unlock(void);

#endif
