/***********************************************************
PROJECT  : I2C interface library
MODULE   : Simple read / write from the command line
TARGET   : Linux/x86
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sat Mar  1 15:42:44 MET 1997
UPDATE   : Wed Apr 10 14:26:38 CEST 2002
VERSION  : 1.6
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

/* works with all the Yail-I2C library/drivers */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "libi2c.h"

typedef enum {NOP=0,VAL,OP_R,OP_W,OP_WR, OP_DEL} tinst;

u_char bufr[256],bufw[256];
u_char countr, countw, address;

char  *usage_string[] = {
"Simple I2C access from the command line",
"Read       > i2csh r <adr> <nb>",
"Write      > i2csh w <adr> <val1> [<val2> ... <valn>]",
"Write/Read > i2csh wr <adr> <nb> <val1> [<val2> ...  <valn>]",
"Delay (x10 ms) > i2csh s <val>",
" Several commands can be chained :",
"    > i2csh w 9E AC 01 w 9E EE s 32 w 9E 22 wr 9E 1 aa",
"    each read command will give a line of output",
NULL};

/* help */
void Usage(char *argv[])
{
   extern char *usage_string[];
   int i;

   printf("** %s **:\n",argv[0]);
   i = 0;
   while (usage_string[i])
      	printf("%s\n",usage_string[i++]);
   exit(-1);
}

void show(u_char *buf,int count)
{
	int i;
	for (i=0;i<count;i++) printf(" %02X",buf[i]);
	printf("\n");
}

int convert(char **arg, u_char *buf,int count)
{
	int i,temp;
	for (i=0;i<count;i++) {
		temp=strtol(arg[i],NULL,16);
		if ((temp<0) || (temp>255)) return -1;
		buf[i]=temp;
	}
	return 0;
}

/* is the next token an instruction ? */
tinst isinstr(char *str) {
    tinst res;
    switch (str[0]) {
        case 'r': res=OP_R; break;
        case 'w': res=OP_W; break;
        case 's': res=OP_DEL; break;
        default:
            if ((str[0]>='0') && (str[0]<='9')) return VAL;
            if ((str[0]>='A') && (str[0]<='F')) return VAL;
            if ((str[0]>='a') && (str[0]<='f')) return VAL;
            return NOP;
    }
    switch (res) {
        case OP_R:
            if (str[1] != '\0') return NOP;
            break;
        case OP_DEL:
            if (str[1] != '\0') return NOP;
            break;
        case OP_W:
            if (str[1] == '\0') return OP_W;
            if (str[1] != 'r')  return NOP;
            if (str[2] != '\0')  return NOP;
            return OP_WR;
         default:
            return NOP;
    }
    return res;
}

/* add a value */
void newval(tinst instr, int rank, u_char val) {
   if (rank==0) {
    address=val;
    return;
   }
   switch (instr) {
        case NOP:
            fprintf(stderr,"No instruction\n");
            exit(-1);

        case OP_R:
            countr=val;
            if (rank>1) {
                fprintf(stderr,"Too much parameters for Read\n");
                exit(-1);
            }
            break;

        case OP_W:
            bufw[rank-1]=val;countw++;
            break;

        case OP_WR:
            if (rank==1) {countr=val;return;}
            bufw[rank-2]=val;countw++;
            break;

	case OP_DEL:
               fprintf(stderr,"Too much parameters for Delay\n");
                exit(-1);     
        default:
            fprintf(stderr,"Can't happen 3 !!!");
            exit(-1);
    }

}

/* execute command */
void doit(tinst instr) {
    int i;
  switch (instr) {
	case OP_DEL:
	       usleep(((int)address)*10000);
	       break;
	case OP_R:
		i=i2c_read(address,bufr,countr) & 0xFF;
		show(bufr,countr);
		if (i<countr) {
			fprintf(stderr,"Truncated read\n");
			exit(-1);
		}
		break;
	case OP_W:
		i=i2c_write(address,bufw,countw) & 0xFF;
		if (i<countw) {
			fprintf(stderr,"Write error\n");
			exit(-1);
		}
		break;	
	case OP_WR:
		i=i2c_write_read(address,bufw,countw,bufr,countr) & 0xFF;
		show(bufr,countr);
		if (i<countw) {
			fprintf(stderr,"Write error\n");
			exit(-1);
		}
		break;
	case NOP:
	       return;
	       break;
        default:
            fprintf(stderr,"Can't happen 2 -> %d\n",instr);
  }
  address=0;
  countr=countw=0;
}

/* main */
int main(int argc, char **argv) {
  tinst instruction,newinst;
  int i,rank;
  long tadr;

  if (argc<4) Usage(argv);

  if (i2c_init(0)) {
   fprintf(stderr,"Unable to initialize i2c bus\n");
   exit(-1);
  }
  
  instruction=NOP;
  rank=0;
  for (i=1;i<argc;i++) {
    switch (newinst=isinstr(argv[i])) {
        case NOP:
            fprintf(stderr,"Unexpected content : %s\n",argv[i]);
            exit(-1);
        case VAL:
            tadr=strtol(argv[i],NULL,16);
            if ((tadr<0) || (tadr>255)) {
  	         perror("Wrong value");
        	exit(-1);
            }
            /* put value in the right place */
    //        fprintf(stderr, "newval %d %d %d\n",instruction,rank,tadr);
            newval(instruction,rank,tadr);
            rank++;
            break;
            
        case OP_R:
        case OP_DEL:
        case OP_W:
        case OP_WR:
            doit(instruction);
            instruction=newinst;
            rank=0;
            break;
            
        default:
            fprintf(stderr,"Can't happen 1 -> %d\n",newinst);
            exit(-1);
    }
  }
  doit(instruction);

  exit(0);
}
