/***********************************************************
PROJECT  : I2C interface library
MODULE   : I2C probing
TARGET   : Linux/x86
COPYRIGHT: (c) 2002 Damien Mercier <Damien.Mercier@grosmi.net>
LICENCE  : LGPL
CREATION : Sat Mar  1 15:42:44 MET 1997
UPDATE   : Thu Dec 16 23:20:53 CET 1999
VERSION  : 1.5
***********************************************************/
/*
This code is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/
/* works with all the Yail-I2C library/drivers */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include "libi2c.h"

static char devi[][128]={
"", /* 0 */
"General call", /* 1 */
"SAA1064 4-digit LED driver", /* 2 */
"PCF8573 Clock/Calendar", /* 3 */
"PCD3311/12 Tone generator DTMF/modem/musical", /* 4 */
"Reserved", /* 5 */
"PCF8200 Voice synthesizer (male or female)", /* 6 */
"PCF8566 96-segment LCD driver 1:1-1:4 Mux", /* 7 */
"PCF8568/69 LCD row/column driver for dot matrix displays", /* 8 */
"PCF8570/71 256x8, 128x8 static RAM", /* 9 */
"PCF8570C 256x8 static RAM", /* 10 */
"PCF8574 I2C bus to 8-bit bus converter", /* 11 */
"PCF8574A I2C bus to 8-bit bus converter", /* 12 */
"PCF8576 160-segment LCD driver 1:1-1:4 Mux", /* 13 */
"PCF8577 64-segment LCD driver 1:1-1:2 Mux", /* 14 */
"PCF8577A 64-segment LCD driver 1:1-1:2 Mux", /* 15 */
"PCF8578/79 Row/column LCD dot-matrix driver", /* 16 */
"PCF8581/82/94 128x8, 256x8, 512x8 EEPROM", /* 17 */
"PCF8583 256x8 RAM with Clock/calendar", /* 18 */
"PCF8591 4-channel 8-bit A/D plus D/A", /* 19 */
"SAA1136 PCM audio indent-word interface", /* 20 */
"SAA1300 5-bit high current driver", /* 21 */
"SAA5243/45/46A Enhanced teletext circuit", /* 22 */
"SAA7191 S-VHS digital multistandard decoder", /* 23 */
"SAA7192 Digital color space converter", /* 24 */
"SAA7199 Digital encoder", /* 25 */
"SAA9020 Field memory controller", /* 26 */
"SAA9051 Digital multi-standard TV-decoder", /* 27 */
"SAA9068 PIP controller", /* 28 */
"SAB3035/36/37 CPU interface for tuning and control", /* 29 */
"SAF1135 Data line decoder", /* 30 */
"TDA4670 Picture signal improvement circuit", /* 31 */
"TDA4680 Video processor", /* 32 */
"TDA8421 Hi-fi stereo audio processor", /* 33 */
"TDA8425 Audio processor w/loudspeaker channel", /* 34 */
"TDA8440 Switch for CTV receivers", /* 35 */
"TDA8442 Interface for color decoders", /* 36 */
"TDA8443 YUV/RGB interface circuit", /* 37 */
"TDA8444 Octuple 6-bit DAC", /* 38 */
"TDA8461 PAL/NTSC color decoder", /* 39 */
"TEA6100 FM/IF and tuning interface", /* 40 */
"TEA6300/6310T Sounf fader controlc circuit", /* 41 */
"TSA5511/12/14 PLL frequency synthetizer for TV", /* 42 */
"TSA6057 Radio tuning PLL frequency synthetizer", /* 43 */
"UMF1009 Frequency synthetizer", /* 44 */
"SAA3028 RC5 -> I2C converter", /* 45 */
"MSM6307 D2B <-> I2C converter", /* 46 */
"DS1621 Digital thermometer and thermostat", /* 47 */
"", /* 48 */
"", /* 49 */
};

static char adr_devices[]={
/***0***  ***2***  ***4***  ***6***  ***8***  ***A***  ***C***  ***E***/
 1, 0, 0, 5, 0, 0, 5, 0, 0, 5, 0, 0, 5, 0, 0, 5, 0, 0, 5, 0, 0, 5, 0, 0, /* 00 02 04 06 08 0A 0C 0E */
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 10 12 14 16 18 1A 1C 1E */
 6,30, 0,22,30, 0,28,30, 0,28,30, 0,26, 0, 0,26, 0, 0,26, 0, 0,26, 0, 0, /* 20 22 24 26 28 2A 2C 2E */
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20, 0, 0, 0, 0, 0, /* 30 32 34 36 38 3A 3C 3E */
11,21,38,11,21,38,11,21,38,11,21,38, 4,11,38, 4,11,38,11,38,46,11,38,46, /* 40 42 44 46 48 4A 4C 4E */
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 50 52 54 56 58 5A 5C 5E */
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* 60 62 64 66 68 6A 6C 6E */
 2,12,13,12,13, 2,12,14, 2,12,15, 2, 8,12,16, 8,12,16, 7,12, 0, 7,12, 0, /* 70 72 74 76 78 7A 7C 7E */
33,41, 0,33,34, 0, 0, 0, 0, 0, 0, 0,31,32,36,23,27,39, 0, 0, 0,23, 0, 0, /* 80 82 84 86 88 8A 8C 8E */
19,35,47,19,35,47,19,35,47,19,35,47,19,35,47,19,35,47,19,35,47,19,35,47, /* 90 92 94 96 98 9A 9C 9E */
 9,17,18, 9,17,18, 9,17, 0, 9,17, 0, 9,17, 0, 9,17, 0, 9,17, 0, 9,17, 0, /* A0 A2 A4 A6 A8 AA AC AE */
10,25, 0,10, 0, 0,10, 0, 0,10, 0, 0,10, 0, 0,10, 0, 0,10, 0, 0,10, 0, 0, /* B0 B2 B4 B6 B8 BA BC BE */
29,42,44,29,40,42,29,42,43,29,42,43, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* C0 C2 C4 C6 C8 CA CC CE */
 3,37, 0, 3,37, 0, 3,37, 0, 3,37, 0,37, 0, 0,37, 0, 0,37, 0, 0,37, 0, 0, /* D0 D2 D4 D6 D8 DA DC DE */
24, 0, 0,24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, /* E0 E2 E4 E6 E8 EA EC EE */
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  /* F0 F2 F4 F6 F8 FA FC FE */
/***0***  ***2***  ***4***  ***6***  ***8***  ***A***  ***C***  ***E***/
};

static inline u_char tohex(u_char x) {
    if (x<10) return x+'0';
    return x-0xA+'A';
}


/* programme principal de test */
int main(int argc, char **argv) {
	char adrutil[128];
        u_char buffer[512];
        u_char *tmpbuf;
	u_short ad,n,p;
        size_t count;
	
	if (i2c_init(0)) {
	   fprintf(stderr,"Unable to initialize i2c bus\n");
	   exit(-1);
	}
	usleep(500000);
	printf("Address probing...");
	tmpbuf=buffer;
/* test des priphriques sur sur le bus */
 	for (ad=0;ad<256;ad+=2) {
 	      if (ad%32 == 0) {
 	          *tmpbuf++='\n';
 	      } else {
 	          *tmpbuf++=' ';
 	      }
 	      n=i2c_test(ad);
 	      switch (n) {
 	          case I2C_OK:
 		     *tmpbuf++=tohex((ad>>4)&0xF);
                     *tmpbuf++=tohex(ad&0xF);
 		     adrutil[ad>>1]=1;
 		     break;
 		  case I2C_TIMEOUT:
 		     *tmpbuf++='#';*tmpbuf++='#';
 		     adrutil[ad>>1]=0;
 		     break;
 		  default:
 		     *tmpbuf++='_';*tmpbuf++='_';
 		     adrutil[ad>>1]=0;
            }
 	}
 	*tmpbuf++='\n';
 	count=tmpbuf-buffer;
       *tmpbuf++='\0';
         printf("%s",buffer);
  	printf("\n## signifie TIMEOUT\n");
  	printf("\n");
	printf("Detected peripherals:\n R  W  (dec) : Possible component(s)\n");
	for (n=0;n<256;n+=2) {
	  if (adrutil[n>>1]) {
	    p=(n>>1)*3;
	    printf("%02X %02X (%03dd):\n\t%s\n\t%s\n\t%s\n",\
	    	n,n+1,n,devi[(int)adr_devices[p]],devi[(int)adr_devices[p+1]],devi[(int)adr_devices[p+2]]);
	  };
	};
	exit(0);
}
